/** @file
 * @brief TZO Service class implementation. This class represents the TZO service.
 *
 *
 *
 * @copyright Intra2net AG
 * @license GPLv2
*/

#include "service_tzo.h"

#include <time.h>
#include <boost/foreach.hpp>
#include <boost/algorithm/string.hpp>

namespace ba = boost::algorithm;

using namespace std;


/**
 * Default Constructor, needed for object serialization.
 */
ServiceTzo::ServiceTzo()
{
}


/**
 * Constructor.
 * @param _hostname The hostname to update
 * @param _login The login name.
 * @param _password The corresponding password.
 */
ServiceTzo::ServiceTzo(const string& _protocol, const string& _hostname, const string& _login, const string& _password, const Logger::Ptr& _logger, const int _update_interval, const int _max_updates_within_interval, const int _dns_cache_ttl, const string& _proxy, const int _proxy_port)
{
    if ( _update_interval == -1 )        // If _update_interval is default po::option_desc (not specified via config)
        set_update_interval(0);              // use default protocol value
    else
        set_update_interval(_update_interval);

    if ( _max_updates_within_interval == -1 )
        set_max_updates_within_interval(0);
    else
        set_max_updates_within_interval(_max_updates_within_interval);

    if ( _dns_cache_ttl == -1 )
        set_dns_cache_ttl(60);
    else
        set_dns_cache_ttl(_dns_cache_ttl);

    set_protocol(_protocol);
    set_hostname(_hostname);
    set_login(_login);
    set_password(_password);
    set_logger(_logger);

    // create http helper class
    HTTPHelper::Ptr _http_help(new HTTPHelper(_logger,_proxy,_proxy_port));
    HTTPHelp = _http_help;
    _http_help.reset();

    BaseUrl = assemble_base_url(get_hostname(),get_login(),get_password());
}


/**
 * Default destructor
 */
ServiceTzo::~ServiceTzo()
{
}


/**
 * Assemble the dyns update url from the given fqhn
 * @param hostname The fqhn hostname to update IP for.
 * @param username The username to use.
 * @param hostname The password to use.
 * @return The assembled update url without IP.
 */
string ServiceTzo::assemble_base_url(const string& fqhn, const string& username, const string& password) const
{
    string base_url;

    base_url = "http://rh.tzo.com";
    base_url.append("/webclient/tzoperl.html?system=tzodns&info=1&tzoname=");
    base_url.append(fqhn);
    base_url.append("&email=");
    base_url.append(username);
    base_url.append("&tzokey=");
    base_url.append(password);
    base_url.append("&ipaddress=");

    return base_url;
}


/**
 * Performs the Service update.
 * @param ip IP Address to set.
 * @return 0 if all is fine, -1 otherwise.
 */
int ServiceTzo::perform_update(const std::string& ip)
{
    string url = BaseUrl;
    url.append(ip);

    long http_status_code = HTTPHelp->http_get(url);

    get_logger()->print_http_status_code(url,http_status_code);

    // HTTP operation completed successful.
    // Now we have to parse the data received by curl,
    // cause http status code is not significant for dyns update errors
    if ( http_status_code == 200 )
    {
        // Get the received http data and parse the status code.
        string curl_data = HTTPHelp->get_curl_data();
        string status_code = parse_status_code(curl_data);

        if ( status_code == "200" )
        {
            return 0;
        }
        else if ( status_code == "401" )
        {
            get_logger()->print_http_not_authorized(url,get_login(),get_password());
        }
        else
        {
            get_logger()->print_update_failure(url,curl_data);
        }
    }
    else
    {
        get_logger()->print_update_failure(url,http_status_code);
    }

    return -1;
}


/**
 * Get the status code from the returned http data
 * @param data The returned http data.
 * @return The status code.
 */
string ServiceTzo::parse_status_code(const string& data) const
{
    list<string> tokens;
    ba::split(tokens,data,boost::is_any_of(" "));
    return tokens.front();
}


/**
 * Serialize function needed by boost/serialization to define which members should be stored as the object state.
 * @param ar Archive
 * @param version Version
 */
template<class Archive>
void ServiceTzo::serialize(Archive & ar, const unsigned int version)
{
    ar & boost::serialization::base_object<Service>(*this);
}
